from datetime import date
from dateutil import rrule
from copy import deepcopy
from random import randrange

DEFAULT_START_DATE = date(2012, 1, 1)
DEFAULT_END_DATE = date(2022, 1, 1)

SYMPTOMS = {
  'Dry Mouth': [],
  'Chills': [],
  'Dehydration': [],
  'Fever': [],
  'Stomach Cramps': [], 
  'Diarrhea': [],
  'Nausea': [],
  'Vomiting': []
}

FOODS = {
  'Food 1': [],
  'Food 2': [],
  'Food 3': [],
  'Food 4': [],
  'Food 5': [], 
  'Food 6': [],
  'Food 7': [],
  'Food 8': []
}

def create_timestamps(start_date, end_date):
  timestamps = []
  for d in rrule.rrule(rrule.MONTHLY, dtstart=start_date, until=end_date):
    timestamps.append(d.timestamp())
  return timestamps


def generate_data(type='symptoms', start_date=DEFAULT_START_DATE, end_date=DEFAULT_END_DATE):
  timestamps = create_timestamps(start_date, end_date)

  data = {}
  if type == 'symptoms':
    data = deepcopy(SYMPTOMS)
  elif type == 'foods':
    data = deepcopy(FOODS)
  else:
    return data
  
  for key in data:
    number = randrange (0, 100)
    for timestamp in timestamps:
      number += randrange(-5, 5)
      number = min(100, number)
      number = max(0, number)
      data[key].append({
        'timestamp': timestamp,
        'value': number
      })
  
  return data

